
Public Class Effects

    ' ------------------------------------------------------------------ FILTER PARAMS
    Friend Enum FilterTypes As Int32
        Lowpass
        Highpass
        Bandpass
        Notch
    End Enum
    Private FilterEnabled As Boolean
    Private FilterType As FilterTypes = FilterTypes.Bandpass
    Private FilterFrequency As Single
    Private FilterResonance As Single
    Private FilterDrive As Single
    Private outLeft1 As Single = 0
    Private outLeft2 As Single = 0
    Private outRight1 As Single = 0
    Private outRight2 As Single = 0

    ' ------------------------------------------------------------------ ECHO PARAMS
    Private EchoEnabled As Boolean
    Private EchoVolume As Single
    Private EchoFeedback As Single
    Private EchoDelayLeft As Single
    Private EchoDelayRight As Single
    Private EchoDelayOffsetLeft As Int32
    Private EchoDelayOffsetRight As Int32

    Private Const DelayLineMask As Integer = 65535
    Private Const DelayLineLength As Integer = DelayLineMask + 1
    Private leftDelayLine(DelayLineLength - 1) As Single
    Private rightDelayLine(DelayLineLength - 1) As Single
    ' --------------------------------------------------- current index for left and right delay lines
    Private index As Integer


    ' ------------------------------------------------------------------ SET FILTER PARAMS 
    Friend Sub SetFilterEnable(ByVal enable As Boolean)
        FilterEnabled = enable
    End Sub
    Friend Sub SetFilterType(ByVal _FilterType As Int32)
        FilterType = CType(_FilterType, FilterTypes)
    End Sub
    Friend Sub SetFilterFrequency(ByVal freq As Single)
        If freq > 1 Then freq = 1
        If freq < 0 Then freq = 0
        FilterFrequency = 0.001F + freq * freq * 0.9F
    End Sub
    Friend Sub SetFilterResonance(ByVal res As Single)
        If res > 1 Then res = 1
        If res < 0 Then res = 0
        FilterResonance = CSng(res ^ 0.2)
    End Sub
    Friend Sub SetFilterDrive(ByVal drv As Single)
        FilterDrive = (0.1F + 2 * drv)
    End Sub

    ' ------------------------------------------------------------------ SET ECHO PARAMS
    Friend Sub SetEchoEnable(ByVal enable As Boolean)
        EchoEnabled = enable
    End Sub
    Friend Sub SetEchoVolume(ByVal vol As Single)
        If vol > 1 Then vol = 1
        If vol < 0 Then vol = 0
        'vol *= vol
        EchoVolume = vol
    End Sub
    Friend Sub SetEchoFeedback(ByVal fbk As Single)
        If fbk > 1 Then fbk = 1
        If fbk < 0 Then fbk = 0
        EchoFeedback = fbk
    End Sub
    Friend Sub SetEchoDelay(ByVal d As Single)
        If d > 1 Then d = 1
        If d < 0 Then d = 0
        d *= d
        If Math.Abs(d - EchoDelayLeft) > 0.00001 Then
            EchoDelayLeft = d
            EchoDelayRight = d * 0.8F
            EchoDelayOffsetLeft = CInt(DelayLineLength * EchoDelayLeft) + 1
            EchoDelayOffsetRight = CInt(DelayLineLength * EchoDelayRight) + 1
            If EchoDelayOffsetLeft >= DelayLineLength Then EchoDelayOffsetLeft = DelayLineLength - 1
            If EchoDelayOffsetRight >= DelayLineLength Then EchoDelayOffsetRight = DelayLineLength - 1
            'index = 0
            'Array.Clear(leftDelayLine, 0, leftDelayLine.Length)
            'Array.Clear(rightDelayLine, 0, rightDelayLine.Length)
        End If
    End Sub


    ' ---------------------------------------------------------------------- NEW 
    Friend Sub New()
        FilterEnabled = False
        FilterType = FilterTypes.Bandpass
        SetFilterFrequency(0.5)
        SetFilterResonance(0.5)
        SetFilterDrive(0.5)
        EchoEnabled = False
        SetEchoVolume(1)
        SetEchoDelay(0.3)
        SetEchoFeedback(0.5)
    End Sub



    Friend Sub Synthesize(ByRef Buffer() As Single, ByVal stereo As Boolean)

        If Not FilterEnabled And Not EchoEnabled Then Return

        ' ---------------------------------------------------------------------- FILTER
        Dim fb As Single = 0.999F * (FilterResonance + FilterResonance / (1.0F - FilterFrequency))
        Dim drv As Single = FilterDrive / (1 + 2 * FilterResonance * FilterResonance)

        ' ---------------------------------------------------------------------- ECHO
        Dim leftIndex As Int32 = (index + EchoDelayOffsetLeft) And DelayLineMask
        Dim rightIndex As Int32 = (index + EchoDelayOffsetRight) And DelayLineMask

        If stereo Then
            ' ------------------------------------------------------------------ STEREO
            Dim vleft As Single
            Dim vright As Single

            If FilterEnabled Then
                For i As Integer = 0 To Buffer.Length - 1 Step 2
                    ' -------------------------------------------------------------- INPUT
                    vleft = Buffer(i) * drv
                    vright = Buffer(i + 1) * drv
                    ' -------------------------------------------------------------- FILTER
                    outLeft1 += FilterFrequency * (vleft - outLeft1 + fb * (outLeft1 - outLeft2))
                    outLeft2 += FilterFrequency * (outLeft1 - outLeft2)
                    outRight1 += FilterFrequency * (vright - outRight1 + fb * (outRight1 - outRight2))
                    outRight2 += FilterFrequency * (outRight1 - outRight2)
                    Select Case FilterType
                        Case FilterTypes.Lowpass : vleft = outLeft2 : vright = outRight2
                        Case FilterTypes.Highpass : vleft = vleft - outLeft2 : vright = vright - outRight2
                        Case FilterTypes.Bandpass : vleft = outLeft1 - outLeft2 : vright = outRight1 - outRight2
                        Case FilterTypes.Notch : vleft = vleft - (outLeft1 - outLeft2) : vright = vright - (outRight1 - outRight2)
                    End Select
                    ' -------------------------------------------------------------- OUTPUT
                    Buffer(i) = vleft
                    Buffer(i + 1) = vright
                Next
            End If

            If EchoEnabled Then
                For i As Integer = 0 To Buffer.Length - 1 Step 2
                    ' -------------------------------------------------------------- INPUT
                    vleft = Buffer(i)
                    vright = Buffer(i + 1)
                    ' -------------------------------------------------------------- Todo - Nan Correction
                    'If Single.IsNaN(leftDelayLine(index)) Then leftDelayLine(index) = 0
                    'If Single.IsNaN(rightDelayLine(index)) Then rightDelayLine(index) = 0
                    ' -------------------------------------------------------------- ECHO
                    leftDelayLine(leftIndex) = vleft + leftDelayLine(index) * EchoFeedback
                    rightDelayLine(rightIndex) = vright + rightDelayLine(index) * EchoFeedback
                    vleft += leftDelayLine(index) * EchoVolume
                    vright += rightDelayLine(index) * EchoVolume
                    index = (index + 1) And DelayLineMask
                    leftIndex = (leftIndex + 1) And DelayLineMask
                    rightIndex = (rightIndex + 1) And DelayLineMask
                    ' -------------------------------------------------------------- OUTPUT
                    Buffer(i) = vleft
                    Buffer(i + 1) = vright
                Next
            End If

        Else
            ' ------------------------------------------------------------------ MONO
            Dim v As Single

            If FilterEnabled Then
                For i As Integer = 0 To Buffer.Length - 1
                    ' -------------------------------------------------------------- INPUT
                    v = Buffer(i) * drv
                    ' -------------------------------------------------------------- FILTER
                    outLeft1 += FilterFrequency * (v - outLeft1 + fb * (outLeft1 - outLeft2))
                    outLeft2 += FilterFrequency * (outLeft1 - outLeft2)
                    Select Case FilterType
                        Case FilterTypes.Lowpass : v = outLeft2
                        Case FilterTypes.Highpass : v = v - outLeft2
                        Case FilterTypes.Bandpass : v = outLeft1 - outLeft2
                        Case FilterTypes.Notch : v = v - (outLeft1 - outLeft2)
                    End Select
                    ' -------------------------------------------------------------- OUTPUT
                    Buffer(i) = v
                Next
            End If

            If EchoEnabled Then
                For i As Integer = 0 To Buffer.Length - 1
                    ' -------------------------------------------------------------- INPUT
                    v = Buffer(i)
                    ' -------------------------------------------------------------- Todo - Nan Correction
                    'If Single.IsNaN(leftDelayLine(index)) Then leftDelayLine(index) = 0
                    ' -------------------------------------------------------------- ECHO
                    leftDelayLine(leftIndex) = v + leftDelayLine(index) * EchoFeedback
                    v += leftDelayLine(index) * EchoVolume
                    index = (index + 1) And DelayLineMask
                    leftIndex = (leftIndex + 1) And DelayLineMask
                    ' -------------------------------------------------------------- OUTPUT
                    Buffer(i) = v
                Next
            End If

        End If
    End Sub

End Class
